#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid.views.employees`` -- Employee Views
"""

from sqlalchemy import and_

import edbob
from edbob.pyramid import grids
from edbob.pyramid.forms import AssociationProxyField
from edbob.pyramid.views import SearchableAlchemyGridView
from edbob.pyramid.views.crud import Crud

import rattail


class EmployeesGrid(SearchableAlchemyGridView):

    mapped_class = rattail.Employee
    route_name = 'employees'
    route_url = '/employees'
    renderer = '/employees/index.mako'
    sort = 'first_name'

    def filter_map(self):
        return self.make_filter_map(
            first_name=grids.search.filter_ilike(edbob.Person.first_name),
            last_name=grids.search.filter_ilike(edbob.Person.last_name),
            phone=grids.search.filter_ilike(edbob.PersonPhone.number))

    def filter_config(self):
        return self.make_filter_config(
            include_filter_first_name=True,
            filter_type_first_name='lk',
            include_filter_last_name=True,
            filter_type_last_name='lk',
            filter_label_phone="Phone Number")

    def sort_map(self):
        return self.make_sort_map(
            first_name=grids.util.sorter(edbob.Person.first_name),
            last_name=grids.util.sorter(edbob.Person.last_name),
            phone=grids.util.sorter(edbob.PersonPhone.number))

    def query(self):
        q = self.make_query()
        q = q.join(edbob.Person)
        q = q.outerjoin(edbob.PersonPhone, and_(
                edbob.PersonPhone.parent_uuid == rattail.Employee.person_uuid,
                edbob.PersonPhone.preference == 1))
        if not self.request.has_perm('employees.edit'):
            q = q.filter(rattail.Employee.status == rattail.EMPLOYEE_STATUS_CURRENT)
        return q

    def grid(self):
        g = self.make_grid()
        g.append(AssociationProxyField('first_name'))
        g.append(AssociationProxyField('last_name'))
        g.configure(
            include=[
                g.first_name,
                g.last_name,
                g.phone.label("Phone Number"),
                ],
            readonly=True)
        return g


class EmployeeCrud(Crud):

    mapped_class = rattail.Employee
    home_route = 'employees.list'

    def fieldset(self, obj):
        fs = self.make_fieldset(obj)
        fs.configure(
            include=[
                fs.person,
                ])
        return fs


def includeme(config):
    EmployeesGrid.add_route(config)
    EmployeeCrud.add_routes(config)
